unit IDE_Info;

interface
Uses Classes ,Windows, SysUtils, Dialogs, StrUtils, cplugInTypes, cpluginInterface, MacroCodeGenerator;

Type
 Cursor = Record
    CsrX: Integer;
    CsrY: Integer;
    end;

Var
  SourceStr: TStringList;
  Labels: TStringList;
  Vars: TStringList;
  DevCore: Integer;

Function GetCursorPos(): Cursor;
Function GetSourceText(): Boolean;
Function GetLabels(): Boolean;
Function GetVars(): Boolean;
Function GetSelectedIDEName(): String;
Function GetSelectedIDECore(): Integer;
Function GetPlugInPath():String;
Procedure PasteTextAtCurrentPos(Text: String; Position: Cursor);

implementation
Function GetPlugInPath():String;
Var
  Buffer: Array of Char;

Begin
  If plgIsConnected then
    begin
      SetLength(Buffer, 256);
      plgGetPathPlugIn(PChar(Buffer),256);
      GetPlugInPath := PChar(Buffer);
    end;
end;

Function GetSelectedIDEName(): String;
Var
  Buffer: Array of Char;
  PageInfo: TPageInfo;

begin
  If plgIsConnected Then
    begin
      setLength(Buffer, 256);                           //
      plgGetPage(PChar(Buffer), 256);                   // Get the file name for the current page
      if plgGetPageInfo(PChar(Buffer), @PageInfo) then  // Get the Page info for the page
      GetSelectedIDEName := PageInfo.DisplayName;                  // Return Name
      end;
  Buffer := Nil;
end;

Function GetSelectedIDECore(): Integer;
Var
  Buffer: Array of Char;
  PageInfo: TPageInfo;

begin
  If plgIsConnected Then
    begin
      setLength(Buffer, 256);                           //
      plgGetPage(PChar(Buffer), 256);                   // Get the file name for the current page
      if plgGetPageInfo(PChar(Buffer), @PageInfo) then  // Get the Page info for the page
      GetSelectedIDECore := PageInfo.DeviceCore;        // Return Core number 12, 14 or 16
      end;
  Buffer := Nil;
end;

Function GetSourceText(): Boolean;
// Gets source string for current page and saves each line in the SourceStr Array.
Var
  Buffer: Array of Char;
  PageInfo: TPageInfo;
  s: String;
  i: Integer;
Begin
  GetSourceText := False;
  If plgIsConnected Then
    begin
      SourceStr := TStringList.Create;
      setLength(Buffer, 256);                           //
      plgGetPage(PChar(Buffer), 256);                   // Get the file name for the current page
      if plgGetPageInfo(PChar(Buffer), @PageInfo) then  // Get the Page info for the page
        begin
          SetLength(Buffer, PageInfo.PageSize);         // Resize the source string to suit page
          plgGetPageText(PChar(Buffer), PageInfo.PageSize); // Get the page text and
          SourceStr.Text := string(Buffer);             // load it inot string list
          // Clean up data - remove blanks and comment lines
          for i := SourceStr.Count - 1 downto 0 do      // start at end as we will be reducing the count.
            begin
              if SourceStr[i] > '' then
                begin
                  SourceStr[i] := StringReplace(SourceStr[i],#9,#32,[rfReplaceAll]);// Replace Tabs with a space
                  s:= leftStr(TrimLeft(SourceStr[i]),1);
                  if  (s = '''') Or (s = ';') then SourceStr.Delete(i); // Remove comment lines
                  end
                else
                  SourceStr.Delete(i);  // remove blank lines
              end;
          GetSourceText := True;
        end;
      end;
  Buffer := nil;  // clear buffer
end;


Function GetLabels(): Boolean;
// Extracts all Labels (Label:) from the source stringlist and places them in the Labels StringList
Var
  i: Integer;
  lblPos: Integer;
  LblStr: String;

begin
  GetLabels := False;
  Labels := TStringList.Create;                // Create the list
  For i := 0 to SourceStr.Count - 1 Do         // Run through all the items in the list
    Begin
      lblPos := Pos(':',SourceStr[i]);                // Look for a possible label
      If lblPos > 0 then                       // If we have found one...
        begin
          lblStr := LeftStr(SourceStr[i],lblPos -1); //Extract the label
          If CheckName(lblStr) then            // See if it conforms to Labels syntax
            begin
              Labels.Add(lblStr);              // OK so add it to the list
              GetLabels := True;               // and set the return flag true
              end;
          end;
      end;
end;


Function GetVars():Boolean;
Var
 i,j : Integer;
 VarLbl, TmpStr: String;
 VarBeg, VarEnd: Integer;

Begin
  GetVars := False;
  Vars := TStringList.Create;
  For i := 0 to SourceStr.Count - 1 Do
    begin
      TmpStr := UpperCase(Trim(SourceStr[i]));      // Make case insensitive
      If Pos('DIM',TmpStr) = 1 then                 // Look for "Dim" at the start of the line
        Begin                                       // If found...
          TmpStr := Trim(SourceStr[i]);             // Get original string back (Case Sensitive)
          VarBeg := 4;                              // Point beyond DIM
          VarEnd := PosEx(' ',TrimLeft(TmpStr),5);  // Find likely end of Var Name (first space)
          If VarEnd > VarBeg then                   // Probably got a Var so
            begin
              VarLbl := Trim(midstr(tmpStr,VarBeg,VarEnd - VarBeg));   // Extract Label
              For j := Length(VarLbl) downto 1 do   // Remove any non-printing characters
                  If MidStr(VarLbl,j,1) <  ' ' then varlbl := LeftStr(VarLbl, j -1);
              Vars.Add(VarLbl);                     // Save variable label in List
              GetVars := True                       // Set return flag
              end;
          end;
      end;
end;

Function GetCursorPos(): Cursor;
//Gets the current cursor position on the selected file.
Var
  CsrX :Cardinal;
  CsrY: Cardinal;

Begin
  If plgIsConnected Then
    Begin
      plgGetPageCursorPos(CsrX, CsrY);
      GetCursorPos.CsrX := CsrX;
      GetCursorPos.CsrY := CsrY;
      end;
end;

Procedure PasteTextAtCurrentPos(Text: String; Position: Cursor);
// Places the text at the insertion point specified by Cursor.
Begin
  If plgIsConnected Then
    plgSetPageTextAt (Position.CsrX, Position.CsrY, PChar(Text));
end;

initialization
  SourceStr := TStringList.Create;

finalization
Begin
  SourceStr.Free;
  Labels.Free;
  Vars.Free;
  end;
end.
