unit MacSourceReadWrite;

interface

Uses
  Classes, Windows, SysUtils, StrUtils, MacroCodeGenerator;

Function ReadMacSource(CurrentFile:String): Integer;
Procedure WriteMacSource(CurrentFile: String; ParamCnt: integer);

implementation


Function ReadMacSource(CurrentFile:String): Integer;
// Reads the Macro Source file, populates the macro Arrays held in Macro Code Generator
// and returns the number of parameters read.

Var
  mRecords: TStrings;
  s: String;
  i, j, k, m, e: Integer;
  ParamCnt: Integer;

// Returns the FNum Field in a comma separated string R
Function ExtractField(R: String; FNum: Integer): String;
Var
  i, St, En: Integer;
Begin
  En := 0; St := 0;
  For i := 1 to FNum Do
     Begin
      St := En + 1;
      En := PosEx(',', R, St);
      If En = 0 then En := Length(R) + 1; // Handle end of Line
      end;
  ExtractField := MidStr(R,St,En-St);
end;

Begin
    mRecords := TStringList.Create ;            // String list holds each record in a string
    Try
      with mRecords Do
        Begin
          LoadFromFile(CurrentFile);            // Populate the string list from the file
          ParamCnt := 0;                        // Reset the parameter count
          For i := 0 to mRecords.Count -1 Do    // Run through all the records
            begin
              Val(ExtractField(mRecords[i],1),j,e);// First field of record defines record type
              If e = 0 then begin
                Case j of
                1: //Record type 1 - Macro Name Description etc....
                   Begin
                      for k := 2 to 7 do
                      begin
                        s:= ExtractField(mRecords[i],k);
                        case k of
                        2: MacroInfo.MacroName := s;
                        3: Begin
                            Val(s,m,e);
                            MacroInfo.RtnMethod := m;
                            end;
                        4: ;//chkTypeConvOn.checked := StrToBool(s);
                        5: Begin
                            Val(s,m,e);
                            MacroInfo.Target := m;
                            end;
                        6: MacroInfo.NoCode := StrToBool(s);
                        7: MacroInfo.Descr := s;
                        end;
                      end;
                    end;
                2: // Record Type 2 - Arguments;
                    begin
                    Inc(paramcnt);
                    for k := 2 to 6 Do
                      Begin
                        s:= ExtractField(mRecords[i],k);
                        Case k of
                        2: Arguments[ParamCnt].ArgName := s;
                        3: Arguments[ParamCnt].ArgVar := s;
                        4: Begin
                            Val(s,m,e);
                            Arguments[ParamCnt].ArgType := m;
                            end;
                        5: Arguments[ParamCnt].ArgOpt := StrToBool(s);
                        6: Arguments[ParamCnt].ArgAlt := StrToBool(s);
                        end;
                      end;
                    end;
                3:// Record Type 3 - Subroutine and Return Info
                  Begin
                    for k := 2 to 8 Do
                      Begin
                      s:= ExtractField(mRecords[i],k);
                      Case k of
                      2: ReturnData.SRName := s;
                      3: ReturnData.CallType:= StrToBool(s);
                      4: ReturnData.JumpType := StrToBool(s);
                      5: ReturnData.RetVar.ArgName := s;
                      6: Begin
                          Val(s,m,e);
                          ReturnData.RetVar.ArgType := m;
                          end;
                      7: ReturnData.RetVar.ArgOpt := StrToBool(s);
                      8: ReturnData.RetVar.ArgAlt := StrToBool(s);
                      end;
                    end;
                  end;
                end;
              end;
            end;
          end;
      finally
        mRecords.Free;        // finished with list so let it go
      end;
    ReadMacSource:= ParamCnt;
end;


Procedure WriteMacSource(CurrentFile: String; ParamCnt: integer);
Var
  mRecords: TStrings;
  s: String;
  i: Integer;

  Function CStr(i: Integer) :String;
    Var s :String;
  Begin
    str(i,s);
    Result := s;
  end;

Begin
  mRecords := TStringList.Create ;            // String list will holds each record in a string
  // Build first record
  With MacroInfo Do Begin
    s := '1,' + MacroName + ',' + CStr(RtnMethod) + ',,' + CStr(Target)
              +  ',' + BoolToStr(NoCode) + ',' + Descr;
    end;
    mRecords.Add(s);
  // Build Arguments records
  for i := 1 to ParamCnt Do begin
    With Arguments[i] Do begin
      s := '2,' + ArgName + ',' + ArgVar + ',' + CStr(ArgType) + ','
                + BoolToStr(ArgOpt) + ',' + BoolToStr(ArgAlt);
      end;
    mRecords.Add(s);
    end;
  // Build Return Record
  With ReturnData Do begin
      s := '3,' + SRName + ',' + BoolToStr(CallType) + ',' + BoolToStr(JumpType) + ','
                + RetVar.ArgName + ',' + CStr(RetVar.ArgType) + ',' + BoolToStr(RetVar.ArgOpt)
                + ',' + BoolToStr(RetVar.ArgAlt);
      end;
    mRecords.Add(s);
  mRecords.SaveToFile(CurrentFile);
end;

end.

