unit MacroCodeGenerator;

Interface

Uses
  Classes, Windows, StrUtils, SysUtils;

Type
   DataTypeDesc = Record
    LabelDesc: String;
    Descr: String;
    Macro: String;
    UIName: String;
    end;

  MacGen = Record
    MacroName: String;
    RtnMethod: Integer;
    Target: Integer;
    NoCode: Boolean;
    Descr: String;
    ParamCnt: Integer;
    end;

  ArgData = Record
    ArgName: String;
    ArgType: Integer;
    ArgVar:  String;
    ArgOpt:  Boolean;
    ArgAlt:  Boolean;
    end;

  RetData = Record
    SRName: String;
    CallType: Boolean;
    JumpType: Boolean;
    RetVar:   ArgData;
    end;

  Function StrDataType(Index, Target: Integer):DataTypeDesc;
  Function CheckParamSet(): Boolean;
  Function CheckName(LblName: String):Boolean;
  Function EvaluateParams(ParamName, VarName: String; NoCode: Boolean): Boolean;
  Function GetMacroCodeRtf(ParamCnt: Integer): TStringList;
  Function GetHintStr(ParamCnt: Integer; HelpFile, TopicID: String ): TStringList;
  Function GetHintRTF(HintStr:TStrings): String;

  Var
  Arguments: array[1..20] of ArgData;
  ReturnData: RetData;
  MacroInfo: MacGen;

implementation

Var
  Code: TStringList;
  Hint: TStringList;

Function EvaluateParams(ParamName, VarName: String; NoCode: Boolean): Boolean;
// Checks all values set and valid for a single parameter
Var
  Rtn :Boolean;

Begin
  Rtn := CheckName(ParamName) ;
  If Rtn = True Then
          If Not NoCode Then Rtn := CheckName(VarName);
EvaluateParams := Rtn;
end;

Function CheckParamSet(): Boolean;
// This checks for consistency and compatible use of the FSR registers.
Var
  i, j: Integer;
  FSRs: Array[1..3] of Boolean;
  ParamStr: String;
  s: Integer;

Begin
  CheckParamSet := True;
  For i := 1 To MacroInfo.ParamCnt do
    begin
      ParamStr := Arguments[i].ArgVar;
    // only check if its a register
      If POS(ParamStr, 'FSR') > 0 Then
        begin
          If (MacroInfo.Target = 2) Then  // 16 bit core provides a 3 FSR registers Make sure each use is unique
            begin
              Val(RightStr(ParamStr, 1),j,s); // Pick up the last charater (should be 0-2)
              If FSRs[j] = True Then
                CheckParamSet := False
              Else
                FSRs[j] := True;
              end
          Else            // 14 bit cores provide one FSR register.
            begin
              If FSRs[1] = True Then
                CheckParamSet := False
              Else
                FSRs[1] := True;
              end;
          end;
      end;
end;

// Returns false if LblName contains illegal characters.
Function CheckName(LblName: String):Boolean;
Var
  i: Integer;
  BadChr: String;
begin
  If lblName > '' then  // Check for characters
    begin
    CheckName := True;
      If LeftStr(lblName, 1) > '@' then  // Check first Char not numeral or a lower asci value
      begin
        For i := 1 to Length(lblName) do  // check each char for illegal ASCII values
          begin
            BadChr := MidStr(lblName,i,1);
            If (BadChr < '0')
            Or ((BadChr > '9') And (BadChr < 'A'))
            Or ((BadChr > 'Z') And (BadChr < '_'))
            Or ((BadChr > '_') And (BadChr < 'a'))
            Or (BadChr > 'z') then
              begin
                CheckName := False;
                Break;
                end;
            end;
          end
        Else
         CheckName := False;
      end
   Else
      CheckName := False;
end;


Function StrDataType(Index, Target: Integer): DataTypeDesc;
Var
  ReturnParam: DataTypeDesc;
Begin
  With ReturnParam do
    If Target > 1 then // PIC 18 series
      Case Index of
      2:  begin
           LabelDesc := 'BYTE,NUM8,NUM16,NUM32,WORD,DWORD,FLOAT';
           Descr := '"a Byte variable or number","a Byte, Word, DWord, Float variable or number"';
           Macro := 'BYTE_BYTE,NUM_BYTE,NUM_BYTE,NUM_BYTE,BYTE_BYTE,BYTE_BYTE,FLOAT_BYTE';
           UIName := 'Byte/Num|Byte';
          end;

      3:  begin
            LabelDesc := 'WORD,NUM16,NUM8,NUM32,BYTE,DWORD,FLOAT';
            Descr := '"a Word variable or number","a Word, Byte, DWord, Float variable or number"';
            Macro := 'WORD_WORD,NUM_WORD,NUM_WORD,NUM_WORD,BYTE_WORD,WORD_WORD,FLOAT_WORD';
            UIName := 'Word/Num|Word';
          end;

      4:  begin
            LabelDesc := 'DWORD,NUM32,NUM16,NUM8,WORD,BYTE,FLOAT';
            Descr := '"a DWord variable or number","a Dword, Word, Byte or Float variable or number"';
            Macro := 'DWORD_DWORD,NUM_DWORD,NUM_DWORD,NUM_DWORD,WORD_DWORD,BYTE_DWORD,FLOAT_DWORD';
            UIName := 'DWord/Num|DWord';
          end;

      5:  begin
            LabelDesc := 'FLOAT,NUM32,NUM16,NUM8,BYTE,WORD,DWORD';
            Descr := '"a Float variable""a Float, Byte, Word, DWord variable or number"';
            Macro := 'FLOAT_FLOAT,NUM_FLOAT,NUM_FLOAT,NUM_FLOAT,BYTE_FLOAT,WORD_FLOAT,DWORD_FLOAT';
            UIName := 'Float/Num|Float';
          end;

      6:  begin
            LabelDesc := 'LABEL,NUM16,NUM8,WORD,N/A,N/A,N/A';
            Descr := '"a pointer to a Label","Pointer to a Label,a Word variable or Number"';
            Macro := '"@  LABEL_WORD","@  LABEL_WORD","@ LABEL_WORD","@  LABEL_WORD",,,';
            UIName := 'Label|N/A';
          end;

      7:  begin
            LabelDesc := 'CHAR';
            Descr := '"part of a Quoted String"';
            Macro := '"@  LABEL_WORD","@ LABEL_WORD","@  LABEL_WORD","@  LABEL_WORD",,,';
            UIName := 'ASCII Char|N/A';
          end;

      9:  begin
            LabelDesc := 'STRING';
            Descr := '"a String Array Pointer"';
            Macro := 'NUM_WORD';
            UIName := 'String|N/A';
          end;

      22: begin
             LabelDesc := 'BYTE_ARRAY_PTR';
             Descr := '"a BYTE array pointer"';
             Macro := 'NUM_WORD';
             UIName := 'Byte Array|N/A';
          end;

      23: begin
             LabelDesc := 'WORD_ARRAY_PTR';
             Descr := '"a WORD array pointer"';
             Macro := 'NUM_WORD';
             UIName := 'Word Array|N/A';
          end;
      Else
          begin
             LabelDesc := '';
             Descr := '"Unrecognised Data Type"';
             Macro := '';
             UIName := 'N/A|N/A';
          end;
      end
  Else
      Case Index of  // PIC 12 and 14 Series.
        2:  begin
              LabelDesc := 'BYTE,NUM8,NUM16,NUM32,WORD,DWORD';
              Descr := '"a Byte variable or number","a Byte, Word, DWord, variable or number"';
              Macro := 'BYTE_BYTE,NUM_BYTE,NUM_BYTE,NUM_BYTE,BYTE_BYTE,BYTE_BYTE';
              UIName := 'Byte/Num|Byte';
            end;

        3:  begin
              LabelDesc := 'WORD,NUM16,NUM8,NUM32,BYTE,DWORD';
              Descr := '"a Word variable or number","a Word, Byte, DWord variable or number"';
              Macro := 'WORD_WORD,NUM_WORD,NUM_WORD,NUM_WORD,BYTE_WORD,WORD_WORD';
              UIName := 'Word/Num|Word';
            end;

        4:  begin
              LabelDesc := 'DWORD,NUM32,NUM16,NUM8,WORD,BYTE';
              Descr := '"a DWord variable or number","a Dword, Word, Byte or variable or number"';
              Macro := 'DWORD_DWORD,NUM_DWORD,NUM_DWORD,NUM_DWORD,BYTE_DWORD,WORD_DWORD';
              UIName := 'DWord/Num|DWord';
            end;

        6:  begin
              LabelDesc := 'LABEL,NUM16,NUM8,WORD,N/A,N/A,N/A';
              Descr := '"a pointer to a Label","Pointer to a Label,a Word variable or number"';
              Macro := '"@ LABEL_WORD","@  LABEL_WORD","@  LABEL_WORD","@  LABEL_WORD",,,';
              UIName := 'Label|N/A';
            end;

        7:  begin
              LabelDesc := 'CHAR';
              Descr := '"part of a Quoted String"';
              Macro := '"@ LABEL_WORD","@  LABEL_WORD","@  LABEL_WORD","@  LABEL_WORD",,,';
              UIName := 'ASCII Char|N/A';
            end;

        22: begin
              LabelDesc := 'BYTE_ARRAY_PTR';
              Descr := '"a BYTE array pointer"';
              Macro := 'NUM_WORD';
              UIName := 'Byte Array|N/A';
            end;

        23: begin
              LabelDesc := 'WORD_ARRAY_PTR';
              Descr := '"a WORD array pointer"';
              Macro := 'NUM_WORD';
              UIName := 'Word Array|N/A';
            end;
      Else
            begin
              LabelDesc := '';
              Descr := '';
              Macro := '';
              UIName := 'N/A|N/A';
            end;
      end;
  StrDataType := ReturnParam;
end;

Function GetMacroCodeRtf(ParamCnt: Integer): TStringList;
Var i, j : Integer;
  ErrString: String;
  TempStr: String;
  cTab: Integer;
  s, t: String;
  ManParamCnt: Integer;
  RtnCnt: Integer;
  TempTypeDesc: DataTypeDesc;
  TempLabel: TStrings;
  TempDescr: TStrings;
  TempMacro: TStrings;
const
  vtTab = #9;

  Function Tab(): String; // Returns tab char at cTab position
  begin
    Tab := DupeString(vtTab, cTab)
  end;

  Function RTab(): String; // Returns tab char at cTab position + 1
  begin
    inc(cTab);
    RTab := DupeString(vtTab, cTab);
    end;

  Function LTab(): String; // Returns tab char at cTab position - 1
  begin
    If cTab > 0 then Dec(cTab);
    LTab := DupeString(vtTab, cTab);
    end;


Begin
  ErrString := '#ERROR "' + MacroInfo.MacroName + ' - ';
  cTab := 0;
// Set up some string lists
  TempLabel := TStringList.Create; TempLabel.Delimiter := ',';
  TempDescr := TStringList.Create; TempDescr.QuoteChar := '"';
  TempMacro := TStringList.Create; TempMacro.Delimiter := ',';

  // Add in the Despription to the start of the Code block.
  Code.Text := MacroInfo.Descr;
  for i := 0 to Code.Count - 1 Do //Preface each line with a comment char '
    Code[i] := '''' + Code[i];

  // Generate First line of Macro - E.g MACRONAME MACRO P1, P2, RETURNVALUE
  tempStr := MacroInfo.MacroName + ' MACRO ';
  ManParamCnt := 0;                           // Reset Mandatory parameter count.
  If ParamCnt > 0 then
    begin
      For i := 1 to ParamCnt do
        begin
          Str(i,s);
          tempStr := TempStr + ' P' + s + ',';
          If Not Arguments[i].ArgOpt then ManParamCnt := i;
        end;
    end;
  If MacroInfo.RtnMethod = 2 Then   //Using Parameteric return
    Begin
      TempStr := TempStr + ' RETURNVALUE';
      If Not ReturnData.RetVar.ArgOpt Then inc(ManParamCnt); // If mandatory return make sure its added to mandatory param count
      RtnCnt := 1;
    end
  else
    begin
      If ParamCnt > 0 Then
        TempStr := LeftStr(TempStr, Length(TempStr) -1);   // remove the last comma and space
      RtnCnt := 0;
    end;
  Code.Add(TempStr);

If ParamCnt > 0 Then
  begin
    // Check for too many args
    Str(ParamCnt + RtnCnt,s);                   // Generate max number of Args as string
    Code.Add ('#IF (PRM_COUNT > ' + s + ')');   // Build > param count test
    Code.Add (RTab + ErrString + 'Too many parameters"');
    Code.Add (LTab + '#ELSE');
    // If at least 1 parameter is required test for minimum paramter count
    If ManParamCnt > 0 then
      begin
        Str(ManParamCnt,s);            // Generate min number of Args as string
        Code.Add (RTab + '#IF (PRM_COUNT < ' + s + ')');
        Code.Add (RTab + errString + 'Too few parameters"');
      end;
    //If no params is a valid option add code to execute macro with no params
    If ManParamCnt = 0 Then
      begin
        code.Add (RTab + '#IF (PRM_COUNT == 0)');
        code.Add (RTab + '"Add additional code for no params here ......."' + LTab);
        inc(cTab);
      end;
    code.Add(LTab + '#ELSE');
    inc(cTab);
  end;

    // Generate Data Type tests
    // Numbers are always allowed take the first 4 entries in the Type Description table
    // If any type allowed take all the entries in the Type Description table

    for i := 1 to ParamCnt Do
      Begin
        TempTypeDesc := StrDataType(Arguments[i].ArgType, MacroInfo.Target);
        TempLabel.Clear; TempLabel.DelimitedText := TempTypeDesc.LabelDesc;
        TempDescr.Clear; TempDescr.DelimitedText := TempTypeDesc.Descr;
        TempMacro.Clear; TempMacro.DelimitedText := TempTypeDesc.Macro;
        If Arguments[i].ArgOpt then begin
          Code.Add(Tab + '#IF (PRM_COUNT > ' + IntToStr(i-1) + ')');
          inc(cTab);
        end;
        TempStr := Tab + '#IF';
        for j := 0 to TempLabel.Count -1 do
          // Generate first test
          begin
            if (TempLabel[j] <> 'N/A') then
              Begin
                Str(i,s);
                TempStr := TempStr + '(PRM_' + s + ' != ' + TempLabel[j] + ')';
                If Not (Arguments[i].ArgAlt) And (j = 3) then break;
                if (j < TempLabel.Count - 1) And (TempLabel[j + 1] <> 'N/A') then
                  TempStr := TempStr + ' && ';
              end;
          end;
        Code.Add(TempStr);
        If Arguments[i].ArgAlt And (TempDescr.Count > 1) then
        TempStr := TempDescr[1] else TempStr := TempDescr[0];
        Code.Add(RTab + ErrString + Arguments[i].ArgName + '(Param' + s + ') Should be ' + TempStr + '"');
        If Arguments[i].ArgOpt then Code.Add(LTab + '#ENDIF');
        Code.Add(lTab + '#ENDIF');
      end;

    If MacroInfo.NoCode then
      Code.Add(Tab + '''Place your code here.....')
    else
    // Transfer any parameters into the defined vars or registers and calling label.
      begin
        If paramCnt > 0 then
          Begin
            for i := 1 to ParamCnt do
              begin
                TempTypeDesc := StrDataType(Arguments[i].ArgType, MacroInfo.Target);
                TempLabel.Clear; TempLabel.DelimitedText := TempTypeDesc.LabelDesc;
                TempMacro.Clear; TempMacro.DelimitedText := TempTypeDesc.Macro;

                If Arguments[i].ArgOpt then begin
                  Code.Add(Tab + '#IF (PRM_COUNT > ' + IntToStr(i-1) + ')');
                  inc(cTab);
                end;

                TempStr := Arguments[i].ArgVar;
                If Length(TempStr) = 1 then TempStr := '_' + TempStr; // Allow for single char vars
                If (LeftStr(TempStr, 3) = 'FSR') And (MacroInfo.Target = 2) then TempStr := TempStr + 'L';  //Adjust Register labelling for PIC18
                for j := 0 to TempLabel.Count - 1 do
                  begin
                    If (TempLabel[j] <> 'N/A') then
                      begin
                        Str(i,s);
                        Code.Add(Tab + '#IF (PRM_' + s + ' == ' + templabel[j] + ')');
                        if (LeftStr(TempStr,3) = 'FSR') then
                          Code.Add(RTab + 'NUM_' + LeftStr(TempStr, 4) + ' P' + s)
                        else
                          Code.Add(RTab + TempMacro[j] + ' P' + s + ', ' + TempStr);
                        Code.Add(LTab + '#ENDIF');
                      end;
                    If Not (Arguments[i].ArgAlt) And (j = 3) then break;
                  end;
                If Arguments[i].ArgOpt then Code.Add(LTab + '#ENDIF');
              end;
          end;
          // Add in the Call to subroutine
          If ReturnData.CallType then
            Code.Add(Tab + 'GoSub ' + ReturnData.SRName)
          else
            code.Add(Tab + 'GoTo ' + ReturnData.SRName);
      end;
  // Finish off the endifs for Input Parameters
  While cTab > 0 do
    Code.Add(LTab + '#ENDIF');

  // Add in the return if required
  If MacroInfo.RtnMethod > 0 then
    begin
    // Determine type of return as param and in line have different names
    if MacroInfo.RtnMethod = 1 then t := 'RETURN' else t := 'LST_PRM';
    // Test for presence of return parameter if its mandatory
    If Not(ReturnData.RetVar.ArgOpt) then
      begin
        Code.Add(Tab + '#IF (' + MacroInfo.MacroName + '_' + t + ' != 1)');
        Code.Add(RTab + ErrString + 'Mandatory return parameter misssing"');
        Code.Add(LTab + '#ELSE');
        inc(cTab);
        end
      else
        begin
          Code.Add(Tab + '#IF (' + MacroInfo.MacroName + '_' + t + ' == 1)');
          inc(cTab);
        end;

    // Test for Return parameter type
    TempTypeDesc := StrDataType(ReturnData.RetVar.ArgType, MacroInfo.Target);
    TempLabel.Clear; TempLabel.DelimitedText := TempTypeDesc.LabelDesc;
    TempDescr.Clear; TempDescr.DelimitedText := TempTypeDesc.Descr;
    TempMacro.Clear; TempMacro.DelimitedText := TempTypeDesc.Macro;
    TempStr := Tab + '#IF';
    for j := 0 to TempLabel.Count - 1 do
      begin
        If (j = 0) Or (j > 3) then
          begin
            TempStr := Tempstr + '(' + t + '_TYPE != ' + tempLabel[j] + ')';
            If Not(ReturnData.RetVar.ArgAlt) And (j = 3) then Break;
            If j < TempLabel.Count -1 then TempStr := TempStr + ' && ';
          end;
      end;
    code.Add(TempStr);

    If ReturnData.RetVar.ArgAlt then TempStr := TempDescr[1] else TempStr := TempDescr[0];
    TempStr := LeftStr(TempStr,Pos(' or number',TempStr) -1);  // remove number option
    Code.Add(RTab + ErrString + 'Return variable should be ' + TempStr + '"');
    Code.Add(LTab + '#ELSE');
    inc(cTab);
    // Now generate the Return code
    If MacroInfo.RtnMethod = 1 then s := '_VAR' else s := '_NAME';
    TempStr := ReturnData.RetVar.ArgName;
    If Length(TempStr) = 1 then TempStr := '-' + TempStr;
    for j := 0 to Templabel.Count -1 do
      begin
        If (j = 0) Or (j > 3) then
          begin
            Code.Add(Tab + '#IF (' + t + '_TYPE == ' + TempLabel[j] + ')');
            Code.Add(RTab + TempMacro[j] + ' ' + TempStr + ', ' + t + s);
            Code.Add(LTab + '#ENDIF');
            If Not ReturnData.RetVar.ArgAlt then break;
          end;
        end;
    code.Add(LTab + '#ENDIF');
  end;
  // Finish off the endifs 
  While cTab > 0 do
    Code.Add(LTab + '#ENDIF');
  //and endm
  Code.Add(LTab + 'ENDM');
// free up the stringlists
TempLabel.Free;
TempDescr.Free;
TempMacro.Free;
// Return the completed code
Result := Code;
end;

Function GetHintStr(ParamCnt: Integer; HelpFile, TopicID: String): TStringList;
Var
  TempStr: String;
  Opt : String;

  Const
  vtTab = #9;

  Function HintNoReturn :String;
    Var
    i : Integer;
    s: String;
    TypeDesc : DataTypeDesc;
    begin
      For i := 1 to ParamCnt do begin
        With Arguments[i] Do begin
          TypeDesc := StrDataType(ArgType, MacroInfo.Target);
          If ArgOpt then Opt := ' Optional ' else Opt := '';
          s := s + Opt + ArgName + ' As ';
          if AnsiContainsText(TypeDesc.LabelDesc ,',') then
            s := s + LeftStr(TypeDesc.LabelDesc, Pos(',', TypeDesc.LabelDesc) -1) + ', '
          Else
            s := s + TypeDesc.LabelDesc + ', ';
        end;
      end;
      s := LeftStr(s,Length(s)-2);   //Remove last comma
      Result := '<b>' + MacroInfo.MacroName + '</b> <i>' + s + '</i>';
    end;


  Function HintReturn: String;
  Var
    s: String;
    TypeDesc: DataTypeDesc;

  Begin
    If MacroInfo.RtnMethod = 1 then
      Result := '<i>Var</i> = ' + HintNoReturn
    else begin
      TypeDesc := StrDataType(ReturnData.RetVar.ArgType, MacroInfo.Target);
      s := TypeDesc.LabelDesc;
      if AnsiContainsText(TypeDesc.LabelDesc ,',') then
        s :=  LeftStr(TypeDesc.LabelDesc, Pos(',', TypeDesc.LabelDesc) -1)
      Else
        s := TypeDesc.LabelDesc;
      Result := HintNoReturn + '<i>ReturnVar As ' + s + '</i>';
    end;
  end;

Begin
  Case MacroInfo.Target of
  0: TempStr := '[dc12, dc14, dc16]';
  1: TempStr := '[dc12, dc14]';
  2: TempStr := '[dc16]';
  end;

  Hint.Clear;
  Hint.Add ('KEY ("' + MacroInfo.MacroName + '")');  // Generate Key
  Hint.Add(vtTab + 'DeviceCore = ' + TempStr);       // Add in Device Type
  Hint.Add(vtTab + 'Highlighter = KeywordUser');     // Add in Highlighter Attribute

  TempStr := vtTab + 'Hint = "';
  If MacroInfo.RtnMethod > 0 then
    If ReturnData.RetVar.ArgOpt then
      // Var = MACRONAME P1, P2 etc... or MACRONAME P1, P2 RETURN etc...
      // MACRONAME P1, P2 etc...
      Hint.Add (TempStr + HintReturn + '<br>' + HintNoReturn + '</i>"')
    else
      // Var = MACRONAME P1, P2 etc... or MACRONAME P1, P2 RETURN etc...
      Hint.Add(TempStr + HintReturn + '</i>"')
  else
    // MACRONAME P1, P2, P3 etc...
    Hint.Add(TempStr + HintNoReturn + '</i>"');
  // Add in Help Link
  if TopicID > '' then TempStr := '"->"' + TopicID + '"' else TempStr := '"';
  if HelpFile > '' then Hint.Add(vtTab + 'Help = "' + HelpFile + TempStr);
  // Add Terminating legend
  Hint.Add('ENDKEY');
  Result := Hint;
end;

Function GetHintRTF(HintStr: TStrings): String;
Var
  Hdr: String;
  s: String;
  
begin
  // Generate RTF header
  hdr := '{\rtf1\ansi\ansicpg1252\deff0\deflang2057{\fonttbl{\f0\fnil\fcharset0 Arial;}}';
  s:= RightStr(HintStr[3], Length(HintStr[3]) - 9); // Trim the Hint = "
  s:= LeftStr(s,Length(s) -1);
  s := ANSIReplaceStr(s, '<b>',  '\b ');      // Convert HTML tags to RTF tags
  s := ANSIReplaceStr(s, '</b>', '\b0 ');
  s := ANSIReplaceStr(s, '<i>',  '\i ');
  s := ANSIReplaceStr(s, '</i>', '\i0 ');
  s := ANSIReplaceStr(s, '<br>', '\par ');
  Result := hdr + '\fs16 ' + s;         // Add header, set font size (8 point) return string
end;

Initialization
begin
  Code := TStringList.Create;
  Hint := TStringList.Create;
end;

Finalization
begin
  code.Free;
  Hint.Free;
end;
end.
